from manim import *
from subprocess import Popen
import os
import shutil


__all__ = ["RstMobject"]


class EmptyTexTemplate(TexTemplate):
    def __init__(self):
        super().__init__()
    
    def _rebuild(self):
        self.body = self.placeholder_text


def create_pandoc_folder():
    if not os.path.exists("pandoc"):
        os.mkdir("pandoc")


def write_rst_file(rst_content: str):
    file_name = os.path.join("pandoc", "temp.rst")
    with open(file_name, "w", encoding="utf-8") as f:
        f.write(rst_content)
    return file_name


def rst_to_tex(rst_content: str):
    create_pandoc_folder()
    file_name = write_rst_file(rst_content)
    tex_file_name = file_name.replace(".rst", ".tex")
    p = Popen(["pandoc", file_name, "-t", "latex", "--standalone", "-o", tex_file_name, file_name])
    p.wait()
    with open(tex_file_name, "r", encoding="utf-8") as f:
        tex_content = f.read()
    tex_content = tex_content.replace("\documentclass[\n]{article}", "\documentclass[preview]{standalone}")
    return tex_content


def clean_up():
    shutil.rmtree("pandoc")

class RstMobject(Tex):
    def __init__(
        self,
        rst_string: str,
        arg_separator="",
        tex_environment=None,
        tex_template=EmptyTexTemplate(),
        **kwargs
    ):
        super().__init__(
            rst_to_tex(rst_string),
            arg_separator=arg_separator,
            tex_environment=tex_environment,
            tex_template=tex_template,
            **kwargs
        )



from manim import *
from .rst_mobject import RstMobject


__all__ = ["MobjectDescription"]


class MobjectDescription(Group):
    def __init__(
        self,
        mobject: Mobject,
        title: str | None = None,
        description: str | None = None,
        rec_config: dict | None = None,
        height: float | None = None,
        buff: float = None,
        **kwargs
    ):
        super().__init__(**kwargs)
        self.mobject = mobject
        self.description = mobject.__doc__ if description is None else description
        examples_idx = self.description.find("Examples")
        if examples_idx != -1:
            self.description = self.description[:examples_idx]
        self.description = self.description.split("\n")
        for i in range(1, len(self.description)):
            self.description[i] = self.description[i].removeprefix(4 * " ")
        self.description = "\n".join(self.description)
        self.description.replace("~.", "")
        self.title = mobject.__class__.__name__ if title is None else title
        self.title_mobject = Tex(f"\\textbf{{{self.title}}}")
        self.description_mobject = RstMobject(self.description).scale_to_fit_height(5)
        self.add(self.title_mobject, self.mobject, self.description_mobject)
        buff = 0.5 if buff is None else buff
        self.arrange(DOWN, buff=buff)
        rec_config = {
            "stroke_color": WHITE,
            "stroke_width": 2,
            "fill_color": BLACK,
            "fill_opacity": 0.5,
            "buff": 0.2,
            "corner_radius": 0.2
        } if rec_config is None else rec_config
        self.add_to_back(SurroundingRectangle(self, **rec_config))
        height = 7 if height is None else height
        self.scale_to_fit_height(height)
